#ifndef TO8_MON_H
#define TO8_MON_H
/*
 * derived from vectrex.h
 * now this compiler is also a Thomson TO8/TO9+ SDK
 * we map some monitor calls here, we don't need a complete C lib, it would be too large for those machines...
 */
/* This file describes library routines that can be used from the
 * C code.
 *
 * Vectrex bios routines are mapped to C-Makros, so that they can be
 * called efficiently.
 *
 * GCC uses certain registers for internal book keeping, we must be
 * aware of these registers, so that we don't clobber them with
 * our asm 'bios' calls.
 *
 * I use makros to check this out and wrap the calls in push/puls if
 * neccesary.
 *
 * The main problem register is the gcc FRAME_POINTER_REGISTER.
 * In the 'usual' gcc m6809 compiler this used to be the U register,
 * I changed that to Y register. The Y register access need in
 * general MORE cycles than the U register. BUT the generated code
 * gets shorter and faster if we do NOT use any framepointer at all.
 * gcc option: -fomit-frame-pointer
 * The problem is, that even if we tell gcc to omit the frame pointer,
 * it seems to have problems (or I did something wrong in the m6809 files)
 * with keeping track of the frame_pointer_register. Even if we omit it,
 * gcc assumes that the frame_pointer_register does only change its contents,
 * if it does so by itself (gcc).
 * So if we know that out asm code changes the frame_pointer_register, we
 * must make sure that when we return to our 'normal' course of action
 * the framepointer is restored to its previous state.
 * The below defines and makros assume that it is possible
 * to switch to another framepointer, but that is ONLY possible if
 * you recompile the compiler (gcc). It expects defines, as to what register
 * is used as framepointer, three possible choices:
 * FRAME_Y (is default)
 * FRAME_X
 * FRAME_U (was default)
 * Y register seems to be a really good choice for vectrex, because it
 * is seldom used by the BIOS, so we don't have to save it often.
 *
 * The gcc option: -mfixed-regs
 * Tells the compiler to assume, that no registers are available over
 * function calls.
 *
 *
 *
 *
 *
 */

// save any registers used in asm when FIXED_REGS_NO_FRAME is set
// BUT the frame pointer.
// even if we say that no frame pointer should be used, gcc thinks
// it should be immune to any changes...
//
// only needed if the asm DOES change the reg...

/* default to FRAME_Y */
#ifndef FRAME_Y
#ifndef FRAME_U
#ifndef FRAME_X
#define FRAME_Y
#endif
#endif
#endif

#ifdef FRAME_Y
#define save_frame_y() asm("pshs \ty \t; save y register");
#define restore_frame_y() asm("puls \ty \t; save y register");
#define save_frame_x() ;
#define restore_frame_x() ;
#define save_frame_u() ;
#define restore_frame_u() ;
#endif
#ifdef FRAME_U
#define save_frame_u() asm("pshs \tu \t; save u register");
#define restore_frame_u() asm("puls \tu \t; save u register");
#define save_frame_x() ;
#define restore_frame_x() ;
#define save_frame_y() ;
#define restore_frame_y() ;
#endif
#ifdef FRAME_X
#define save_frame_x() asm("pshs \tx \t; save x register");
#define restore_frame_x() asm("puls \tx \t; save x register");
#define save_frame_y() ;
#define restore_frame_y() ;
#define save_frame_u() ;
#define restore_frame_u() ;
#endif

#define true (1==1)
#define false (1==0)

static unsigned char __A;
static unsigned char __B;
static short __D;
static short __X;
static short __Y;
static short __U;
/* FRAME_POINTER destroys D register... might use a push instead... */
/* FRAME_POINTER is frame pointer, that <should> not be spilled by us! */
/* we don't spill any registers
   since we 'know' what we do! (otherwise allways the set register
                                would be spilled)
   attention with FRAME_POINTER register!   */
#define SET_A_REG_IM(a) asm("lda %0"::"K" ((unsigned char)a) );
#define SET_B_REG_IM(b) asm("ldb %0"::"K" ((unsigned char)b) );
#define SET_D_REG_IM(d) asm("ldd %0"::"L" ((unsigned short)d) );
#ifdef FRAME_X
#define SET_X_REG_IM(x) asm("tfr \td,x"::"d" ((unsigned short)x):"d" );
#else
#define SET_X_REG_IM(x) asm("ldx %0"::"L" ((unsigned short)x) );
#endif
#ifdef FRAME_Y
#define SET_Y_REG_IM(y) asm("tfr \td,y"::"d" ((unsigned short)y):"d" );
#else
#define SET_Y_REG_IM(y) asm("ldy %0"::"L" ((unsigned short)y) );
#endif
#ifdef FRAME_U
#define SET_U_REG_IM(u) asm("tfr \td,u"::"d" ((unsigned short)u):"d" );
#else
#define SET_U_REG_IM(u) asm("ldu %0"::"L" ((unsigned short)u) );
#endif
/* we don't spill any registers
   since we 'know' what we do! (otherwise allways the set register
                                would be spilled)
   attention with FRAME_POINTER register!   */
#define SET_A_REG(a) asm(""::"A" ((unsigned char)a));
#define SET_B_REG(b) asm(""::"B" ((unsigned char)b));
#define SET_D_REG(d) asm(""::"d" ((unsigned short)d));
#ifdef FRAME_X
#define SET_X_REG(x) asm("tfr \td,x"::"d" ((unsigned short)x):"d");
#else
#define SET_X_REG(x) asm(""::"x" ((unsigned short)x));
#endif
#ifdef FRAME_Y
#define SET_Y_REG(y) asm("tfr \td,y"::"d" ((unsigned short)y):"d");
#else
#define SET_Y_REG(y) asm(""::"y" ((unsigned short)y));
#endif
#ifdef FRAME_U
#define SET_U_REG(u) asm("tfr \td,u"::"d" ((unsigned short)u):"d");
#else
#define SET_U_REG(u) asm(""::"u" ((unsigned short)u));
#endif

/* FRAME_POINTER destroys D register... might use a push instead... */
/* FRAME_POINTER is frame pointer, that <should> not be spilled by us! */
#define GET_A_REG() asm("":"=A" ((unsigned char) __A));
#define GET_B_REG() asm("":"=B" ((unsigned char) __B));
#define GET_D_REG() asm("":"=d" ((unsigned short) __D));
#ifdef FRAME_X
#define GET_X_REG() asm("tfr \tx,d":"=d" ((unsigned short) __X)::"d");
#else
#define GET_X_REG() asm("":"=x" ((unsigned short) __X));
#endif
#ifdef FRAME_Y
#define GET_Y_REG() asm("tfr \ty,d":"=d" ((unsigned short) __Y)::"d");
#else
#define GET_Y_REG() asm("":"=y" ((unsigned short) __Y));
#endif
#ifdef FRAME_U
#define GET_U_REG() asm("tfr \tu,d":"=d" ((unsigned short) __U)::"d");
#else
#define GET_U_REG() asm("":"=u" ((unsigned short) __U));
#endif

#define read_ram(mem) ((unsigned char)(*((volatile unsigned char *) ((void *)((volatile unsigned short)(mem))))))
#define write_ram(mem, value) ((*((volatile unsigned char *)(mem))) = ((volatile unsigned char)(value)))

/* NOTE:
 * while calculating address, operands, values... allways make sure
 * D is calculated last, since D is the general Data register which might
 * be destroyed by evaluating other expressions.
 *
 * Likewise, allway calculate A prior to B, since B is the
 * 'more general' register. It is used for every byte evaluation, since
 * there is only one DATA register (D = A+B).
 * If a Data expression is byte size, allways register B is used!
 *
 * clobber constraints seem not to work for FRAME_REGISTER -> save FRAME_REGISTER yourself!
 *
 * !!!!
 erwperw
 */

/* ...  */
#define to8mon_putc(car) { \
                     asm("jsr \t0HE803 \t; putc"\
					 ::"B" ((unsigned char) (car))\
					 :"b"); \
					 }
                     
/* ... */

#ifdef FRAME_U
#define print_str(x,y,string) {save_frame_u() \
                              asm("leau \t,%02 \t; get string address to u\n\tjsr \t0HF37A \t; print_str_d"\
                                  ::"A" ((unsigned char)(y)),\
                                    "B" ((unsigned char)(x)),\
                                    "x" ((unsigned char *)string)\
                                   :"a","b","d","x","u"); \
                              restore_frame_u()}
#else
#define print_str(x,y,string) {save_frame_x() \
                              asm("jsr \t0HF37A \t; print_str_d"\
                                  ::"A" ((unsigned char)(y)),\
                                    "B" ((unsigned char)(x)),\
                                    "u" ((unsigned char *)string)\
                                   :"a","b","d","x","u"); \
                              restore_frame_x()}
#endif

#define set_dp(dp) asm("tfr \tb,dp \t; set dp to b "::"B" ((unsigned char)(dp)):"b","dp")
/* ... dp -> c8 */
#define set_dp_c8() set_dp(0xC8)
/* ... dp -> d0 */
#define set_dp_d0() set_dp(0xD0)
/* ... */

// vectrex 'C' - lib
// extern unsigned char random();
#endif TO8_MON_H



/*

Constraint classes:
    "q"             * 8-bit (byte (HI)) data (a, b, D)
    "d"             * 16-bit (word (SI)) data (only D)
    "a"             * 16-bit addresses S, PC, X, Y, U
    "g"             * general S, PC, X, Y, U , A, B, D
                    * + typical gcc contraints
    "A"             * A register
    "B"             * B register
    "x"             * x register (when framepointer mostly forbidden due to framepointer use...)
    "y"             * y register (when framepointer mostly forbidden due to framepointer use...)
    "u"             * u register (when framepointer mostly forbidden due to framepointer use...)

--------------------------------------------------------------
   The letters I, J, K, L and M in a register constraint string
   can be used to stand for particular ranges of immediate operands.
   This macro defines what the ranges are.
   C is the letter, and VALUE is a constant value.
   Return 1 if VALUE is in the range specified by C.

   For the 6809, J, K, L are used for indexed addressing.
   `I' is used for shift counts which can only be equal to 1.
   `J' is used for the 5-bit offsets.
   `K' is used for the 8-bit offsets.
   `L' is used for the range of signed numbers that fit in 16 bits.
--------------------------------------------------------------


Clobber register names:
"d", "x", "y", "u", "s", "p","a", "b", "cc","dp"


   3. It is possible that if an input dies in an insn, reload might
      use the input reg for an output reload.  Consider this example:

                asm ("foo" : "=t" (a) : "f" (b));

      This asm says that input B is not popped by the asm, and that
      the asm pushes a result onto the reg-stack, ie, the stack is one
      deeper after the asm than it was before.  But, it is possible that
      reload will think that it can use the same reg for both the input and
      the output, if input B dies in this insn.

        If any input operand uses the "f" constraint, all output reg
        constraints must use the "&" earlyclobber.

      The asm above would be written as

                asm ("foo" : "=&t" (a) : "f" (b));


*/
